#!/usr/bin/perl
#
# tpm2deb-common.pl
# machinery to create debian packages from TeX Live depot
# (c) 2005, 2006 Norbert Preining
#
# $Id: tpm2debcommon.pm 3127 2007-10-22 11:52:13Z preining $
#
# configuration is done via the file tpm2deb.cfg
#
#
# please see the README file in the pkg-texlive svn repository of the
# debian-tex project on alioth.debian.org

package tpm2debcommon;
use Exporter ();
@ISA = qw( Exporter );
@EXPORT_OK = qw (
    
    %TeXLive 

	build_data_hash load_collection_tpm_data check_consistency
	initialize_config_file_data is_blacklisted get_all_files 
	get_all_executes myopen mycopy ismember populate_TpmData_from_dump
    dump_tpm_data store_tpm
);
@EXPORT = @EXPORT_OK;

# keys(%TeXLive{$package}) = extra_format, uploaders, section,
#       standards, build_dep_indep, build_dep, priority, description, title,
#       license, remotefiles, sourcefiles, runfiles, docfiles, binfiles,
#       realtype, type, replaces, provides, recommends, suggests,
#       conflicts, depends, includedpackages, executes
#
# keys(%Config) = 
#                 add_execute, title, description, depends, conflicts,
#                 suggests, recommends provides, maintainer

use strict;
no strict 'refs';
use warnings;
no warnings 'uninitialized';


#use Strict;
use Getopt::Long;
use File::Basename;
use File::Copy;
use File::Path;
use File::Temp qw/ tempfile tempdir /;
use Storable;
## not needed, atm we are calling eperl binary 
# use Parse::ePerl;
#use XML::DOM;
use Cwd;
#use FileUtils qw(canon_dir cleandir make_link newpath member
#		 normalize substitute_var_val dirname diff_list remove_list
#		 rec_rmdir sync_dir walk_dir start_redirection stop_redirection);
use Tpm;

# for debuging
# use Data::Dumper;

#

our (%TeXLive);
our (%Config,%TpmData);
our %TexmfTreeOfType = ( "TLCore" => "texmf",
 						"Documentation" => "texmf-doc",
 						"Package" => "texmf-dist");
our @TpmCategories = keys %TexmfTreeOfType;
our %TypeOfTexmfTree = reverse %TexmfTreeOfType;

# pre set $opt_master to ./LocalTPM which contains also the Tools dir
# this is set in the main script, and changed with commandline option.
# should it maybe be deleted here?

my $opt_debug;
my $opt_onlyscripts;
my $Master;
sub use_global_vars {
  $opt_debug = $main::opt_debug;
  $opt_onlyscripts = $main::opt_onlyscripts;
  $Master = $main::Master;
}


sub populate_TpmData_from_dump {
  my $datafile = $_[0];
  my $tpmdataref = retrieve($datafile);
  %TpmData = %{$tpmdataref};
}

sub build_data_hash {
	# local functions
	sub get_type {
		my ($type_slash_tpmname) = @_;
		$type_slash_tpmname =~ m#^(.*)/#;
		return($1);
	}
	sub get_name {
		my ($type_slash_tpmname) = @_;
		$type_slash_tpmname =~ m#/(.*)$#;
		return($1);
	}
	sub tpm2debname {
		my @list_of_debnames =();
		foreach my $tpm_spec (@_) {
			my $tpm_name = get_name($tpm_spec);
			my $tpm_type = get_type($tpm_spec);
# 			my $realtype = $subtype;
			if (defined($Config{'name_mapping'}{$tpm_name})) {
				push @list_of_debnames, $Config{'name_mapping'}{$tpm_name};
			} else {
				push @list_of_debnames, $tpm_name;
			};
		}
		return(@list_of_debnames);
	}
	# start real work
	print "Building data hash ...\n";
	#
	# we now build the information used for building the packages
	# this should merge the tpm data and the config file data
	# after this we check on consistency
	#
	# The data will be stored in %TeXLive{'binary'}{$package}{'item'}
	# where item is in: depend, suggest, conflict, execute, description
	# title, packages, docfiles, binfiles, runfiles, sourcefiles,
	# remotefiles, tltype, license for binary packages, and in
	# %TeXLive{'source'}{$package}{'item'} .... for source packages.
	#
	# Note that the keys in %TeXLive are *different* package names than
	# the collection names!
	#
	# First the binary part
	foreach my $tl_type ('TLCore', 'Documentation', 'Package') {
		foreach my $coll (keys %{$TpmData{$tl_type}}) {
			if (is_blacklisted($coll,"")) { next; }
			my ($bin_pkg) = tpm2debname("$tl_type/$coll");
			my $realtype = $tl_type;
			if ($tl_type eq "TLCore") {
				if ($coll =~ /^bin-(.*)$/) { $realtype = "Package"; }
				if ($coll =~ /^hyphen-(.*)$/) { $realtype = "Package"; }
				if ($coll =~ /^lib-(.*)$/) { $realtype = "Package"; }
			}
			$TeXLive{'binary'}{$bin_pkg}{'type'}        = $realtype;
			$TeXLive{'binary'}{$bin_pkg}{'realtype'}    = $tl_type;
			if ($tl_type eq "Documentation") { $TeXLive{'binary'}{$bin_pkg}{'type'} = "Package"; }
			$TeXLive{'binary'}{$bin_pkg}{'binfiles'}    = $TpmData{$tl_type}{$coll}{'BinFiles'};
			$TeXLive{'binary'}{$bin_pkg}{'docfiles'}    = $TpmData{$tl_type}{$coll}{'DocFiles'};
			$TeXLive{'binary'}{$bin_pkg}{'runfiles'}    = $TpmData{$tl_type}{$coll}{'RunFiles'};
			$TeXLive{'binary'}{$bin_pkg}{'sourcefiles'} = $TpmData{$tl_type}{$coll}{'SourceFiles'};
			$TeXLive{'binary'}{$bin_pkg}{'remotefiles'} = $TpmData{$tl_type}{$coll}{'RemoteFiles'};
			$TeXLive{'binary'}{$bin_pkg}{'license'}     = $TpmData{$tl_type}{$coll}{'License'};
			# items that can be overwritten by the configuration file
			$TeXLive{'binary'}{$bin_pkg}{'title'} =
				$Config{'title'}{$bin_pkg} ? 
					$Config{'title'}{$bin_pkg} : 
					$TpmData{$tl_type}{$coll}{'Title'};
			$TeXLive{'binary'}{$bin_pkg}{'description'} =
				$Config{'description'}{$bin_pkg} ?
					$Config{'description'}{$bin_pkg} : 
					$TpmData{$tl_type}{$coll}{'Description'};
			#
			# executes
			#
			my @installation = @{$TpmData{$tl_type}{$coll}{'Installation'}};
			my @executes = ();
			if (@installation) {
				for (my $i = 0; $i <= $#installation; $i++) {
					my %execute = %{$installation[$i]};
					my $execute = "";
					foreach my $attr (sort keys %execute) {
						$execute .= "$attr=$execute{$attr} ";
					}
					if (grep(/$execute/,@executes) == 0) {
						push (@executes, $execute);
					}
				}
			}
			if (defined($Config{'add_execute'}{$bin_pkg})) {
				foreach my $e (@{$Config{'add_execute'}{$bin_pkg}}) {
					if (grep(/$e/,@executes) == 0) {
						push (@executes, $e);
					}
				}
			}
			$TeXLive{'binary'}{$bin_pkg}{'executes'} = \@executes;
			#
			# included packages
			#
			my @requires = @{$TpmData{$tl_type}{$coll}{'Package'}};
			my @packs = ();
			foreach my $required_tpm (@requires) {
# 				my $st = get_type($required_tpm);
				my $tpm_name = get_name($required_tpm);
				#
				# if the package is moved somewhere else, ignore it
				if (defined($Config{'moved_to'}{$tpm_name})) { 
					next; 
				}
				if (!is_blacklisted($tpm_name,$bin_pkg)) { push @packs, $tpm_name; } 
			}
			if (defined($Config{'extra_packages'}{$bin_pkg})) {
				push @packs, @{$Config{'extra_packages'}{$bin_pkg}};
			}
			$TeXLive{'binary'}{$bin_pkg}{'includedpackages'} = \@packs;
			#
			# depends
			#
			# we have to collect the depends from the config file and the 
			# direct tpm dependencies
			my @depends = tpm2debname(@{$TpmData{$tl_type}{$coll}{'TLCore'}});
			if ($bin_pkg ne "texlive-common") {
				push @depends, "texlive-common (>= $TeXLive{'all'}{'tl_common_version'})";
			}
			#
			if (defined($Config{'depends'}{$bin_pkg})) {
				push @depends, @{$Config{'depends'}{$bin_pkg}};
			}
			# what about ${shlibs:Depends}
			$TeXLive{'binary'}{$bin_pkg}{'depends'} = \@depends;
			#
			# other relations
			#
			if (defined($Config{'conflicts'}{$bin_pkg})) {
				$TeXLive{'binary'}{$bin_pkg}{'conflicts'} = [ @{$Config{'conflicts'}{$bin_pkg}} ];
			}
			if (defined($Config{'suggests'}{$bin_pkg})) {
				$TeXLive{'binary'}{$bin_pkg}{'suggests'} = [ @{$Config{'suggests'}{$bin_pkg}} ];
			}
			if (defined($Config{'recommends'}{$bin_pkg})) {
				$TeXLive{'binary'}{$bin_pkg}{'recommends'} = [ @{$Config{'recommends'}{$bin_pkg}} ];
			}
			if (defined($Config{'provides'}{$bin_pkg})) {
				$TeXLive{'binary'}{$bin_pkg}{'provides'} = [ @{$Config{'provides'}{$bin_pkg}} ];
			}
			if (defined($Config{'replaces'}{$bin_pkg})) {
				$TeXLive{'binary'}{$bin_pkg}{'replaces'} = [ @{$Config{'replaces'}{$bin_pkg}} ];
			}
			if (defined($Config{'bin-section'}{$bin_pkg})) {
				$TeXLive{'binary'}{$bin_pkg}{'section'} = $Config{'bin-section'}{$bin_pkg};
			}
		}
	}
	# we do the doc splitting now
	foreach my $coll (keys %{$TpmData{'TLCore'}}) {
		if (is_blacklisted($coll,"")) { next; }
		my ($bin_pkg) = tpm2debname("TLCore/$coll");
		if (defined($Config{'docsplitting'}{$bin_pkg})) {
			my $doc_pkg = "$bin_pkg-doc";
			# first creat new bin package and add doc files

			$TeXLive{'binary'}{$doc_pkg}{'section'} = "doc" ; 
			$TeXLive{'binary'}{$doc_pkg}{'type'}        = 'TLCore';
			$TeXLive{'binary'}{$doc_pkg}{'realtype'}    = 'TLCore';
			# we don't want to move man pages!!!
			my @p = ();
			my @pd = ();
			foreach my $f (@{$TeXLive{'binary'}{$bin_pkg}{'docfiles'}}) {
				# the following is a hack but necessary
				# we do move some files from doc dirs to non doc dirs using
				# mapping;orig;move;target
				# but these files are listed as doc-files we DONT want to move!
				# In fact we should check for a pair
				# 	$f:xxxx
				# in @{$TeXLive{'all'}{'filemappings'}} and check that xxxx is 
				# NOT a doc file ... but who cares ...
				if (defined($TeXLive{'all'}{'file_map_actions'}{$f}) && 
							($TeXLive{'all'}{'file_map_actions'}{$f} eq "move")) {
					push @p, $f;
				} else {
					if ($f =~ m;texmf[^/]*/doc/man/man.*/.*;) {
						push @p, $f;
					} else {
						push @pd, $f;
					}
				}
			}
			$TeXLive{'binary'}{$doc_pkg}{'docfiles'}    = [ @pd ];
			$TeXLive{'binary'}{$bin_pkg}{'docfiles'} = [ @p ];
			$TeXLive{'binary'}{$doc_pkg}{'license'}     = $TpmData{'TLCore'}{$coll}{'License'};
			$TeXLive{'binary'}{$doc_pkg}{'title'} = "Documentation files for $bin_pkg";
			$TeXLive{'binary'}{$doc_pkg}{'description'} = "This package provides the documentation for $bin_pkg";
			# what else do we have to set here ????
			# add a recommends
			$TeXLive{'binary'}{$bin_pkg}{'recommends'} = [ @{$TeXLive{'binary'}{$bin_pkg}{'recommends'}}, "$bin_pkg-doc" ];
			# add source mapping
			my $srcpkg = $TeXLive{'binary'}{$bin_pkg}{'source_package'};
			$TeXLive{'source'}{$srcpkg}{'binary_packages'} = [ @{$TeXLive{'source'}{$srcpkg}{'binary_packages'}}, $doc_pkg ];
			#
			# necessary relations from the config file
			#
			if (defined($Config{'conflicts'}{$doc_pkg})) {
				$TeXLive{'binary'}{$doc_pkg}{'conflicts'} = [ @{$Config{'conflicts'}{$doc_pkg}} ];
			}
			if (defined($Config{'suggests'}{$doc_pkg})) {
				$TeXLive{'binary'}{$doc_pkg}{'suggests'} = [ @{$Config{'suggests'}{$doc_pkg}} ];
			}
			if (defined($Config{'recommends'}{$doc_pkg})) {
				$TeXLive{'binary'}{$doc_pkg}{'recommends'} = [ @{$Config{'recommends'}{$doc_pkg}} ];
			}
			if (defined($Config{'provides'}{$doc_pkg})) {
				$TeXLive{'binary'}{$doc_pkg}{'provides'} = [ @{$Config{'provides'}{$doc_pkg}} ];
			}
			if (defined($Config{'replaces'}{$doc_pkg})) {
				$TeXLive{'binary'}{$doc_pkg}{'replaces'} = [ @{$Config{'replaces'}{$doc_pkg}} ];
			}
			#
			# now we have to move all the DocFiles of all INCLUDED
			# package/tpms into $bin_pkg and remove them from the original
			# packages
			#
			# For an explanation of the code see above!
			foreach my $incpkg (@{$TeXLive{'binary'}{$bin_pkg}{'includedpackages'}}) {
				my @p = ();
				my @pd = ();
				foreach my $f (@{$TeXLive{'binary'}{$incpkg}{'docfiles'}}) {
					if (defined($TeXLive{'all'}{'file_map_actions'}{$f}) && 
								($TeXLive{'all'}{'file_map_actions'}{$f} eq "move")) {
						push @p, $f;
					} else {
						if ($f =~ m;texmf[^/]*/doc/man/man.*/.*;) {
							push @p, $f;
						} else {
							push @pd, $f;
						}
					}
				}
				$TeXLive{'binary'}{$doc_pkg}{'docfiles'}    = [ @{$TeXLive{'binary'}{$doc_pkg}{'docfiles'}}, @pd ];
				$TeXLive{'binary'}{$incpkg}{'docfiles'} = [ @p ];
			}
		}
	}
	#
	# Now for the source part
	#
	for my $srcpkg (@{$TeXLive{'all'}{'sources'}}) {
		$TeXLive{'source'}{$srcpkg}{'uploaders'}   = 
			$Config{'uploaders'}{$srcpkg}     ? 
				$Config{'uploaders'}{$srcpkg} : $TeXLive{'all'}{'uploaders'};
		$TeXLive{'source'}{$srcpkg}{'maintainer'}  = 
			$Config{'maintainer'}{$srcpkg}    ? 
				$Config{'maintainer'}{$srcpkg} : $TeXLive{'all'}{'maintainer'};
		$TeXLive{'source'}{$srcpkg}{'priority'} = 
			$Config{'priority'}{$srcpkg};
		$TeXLive{'source'}{$srcpkg}{'build_dep'} = 
			$Config{'build_dep'}{$srcpkg};
		$TeXLive{'source'}{$srcpkg}{'build_dep_indep'} =
			$Config{'build_dep_indep'}{$srcpkg};
		$TeXLive{'source'}{$srcpkg}{'standards'} =
			$Config{'standards'}{$srcpkg};
 		$TeXLive{'source'}{$srcpkg}{'section'} =
			$Config{'section'}{$srcpkg};
	}
	# we let texlive-common CONFLICT with all texlive packages << then the 
	# values set in latest-version
	my @conflictpkgs = ();
	foreach my $source_package (@{$TeXLive{'all'}{'sources'}}) {
		foreach my $bin_pkg (@{$TeXLive{'source'}{$source_package}{'binary_packages'}}) {
			push @conflictpkgs, "$bin_pkg (<< $TeXLive{'source'}{$source_package}{'latest_version'})";
		}
	}
	# finally we let the package "texlive-full" depend on all texlive-* packages
	my @allpkgs = ();
	foreach my $source_package (@{$TeXLive{'all'}{'sources'}}) {
		foreach my $bin_pkg (@{$TeXLive{'source'}{$source_package}{'binary_packages'}}) {
			my $addthis = 1;
			foreach my $depends_not (@{$Config{'depends_not'}{'texlive-full'}}) {
				$addthis = 0 if ($bin_pkg eq $depends_not);
			};
# 			next if ($bin_pkg eq "texlive-full");
# 			next if ($bin_pkg eq "texlive");
# 			next if ($bin_pkg eq "texlive-lang-all");
# 			next if ($bin_pkg eq "tetex-bin");
# 			next if ($bin_pkg eq "tetex-base");
# 			next if ($bin_pkg eq "tetex-extra");
			$addthis && push @allpkgs, "$bin_pkg (>= $TeXLive{'source'}{$source_package}{'latest_version'})";
		}
	}

	# Additional (meta)packages
	$TeXLive{'all'}{'meta_packages'} = [ @{$Config{'add_packages'}} ];
	foreach my $meta_package (@{$Config{'add_packages'}}) {
		$TeXLive{'binary'}{$meta_package}{'type'} = "TLCore" ;

		# Dependencies
		if ($meta_package eq "texlive-full") {
			$TeXLive{'binary'}{$meta_package}{'depends'}   = [ @{$Config{'depends'}{$meta_package}}, @allpkgs ];
		} elsif ($meta_package eq "texlive-lang-all") {
			my @foo = ();
			foreach my $a (@allpkgs) {
				if ($a =~ /^texlive-lang-/) { push @foo, $a; }
			}
			$TeXLive{'binary'}{$meta_package}{'depends'}   = [ @{$Config{'depends'}{$meta_package}}, @foo ];
		} else {
			$TeXLive{'binary'}{$meta_package}{'depends'}   = [ @{$Config{'depends'}{$meta_package}} ];
		};
		$opt_debug && print STDERR "metapackage: $meta_package, Depends: @{$TeXLive{'binary'}{$meta_package}{'depends'}}\n";
		$TeXLive{'binary'}{$meta_package}{'suggests'}    = [ @{$Config{'suggests'}{$meta_package}} ];
		$TeXLive{'binary'}{$meta_package}{'recommends'}  = [ @{$Config{'recommends'}{$meta_package}} ];
		$TeXLive{'binary'}{$meta_package}{'replaces'}    = [ @{$Config{'replaces'}{$meta_package}} ];
		if ($meta_package eq "texlive-common") {
			$TeXLive{'binary'}{$meta_package}{'conflicts'}   = [ @{$Config{'conflicts'}{$meta_package}}, @conflictpkgs ];
		} else {
			$TeXLive{'binary'}{$meta_package}{'conflicts'}   = [ @{$Config{'conflicts'}{$meta_package}} ];
		}
	  
		# Short and long description
		$TeXLive{'binary'}{$meta_package}{'title'}       = $Config{'title'}{$meta_package};
		$TeXLive{'binary'}{$meta_package}{'description'} = $Config{'description'}{$meta_package};
		if (defined($Config{'bin-section'}{$meta_package})) {
			$TeXLive{'binary'}{$meta_package}{'section'} = $Config{'bin-section'}{$meta_package};
		}
	}
}

sub load_collection_tpm_data {
	# local functions
	sub trim {
		my ($str) = @_;
		$str =~ s/^[\n\s]+//;
		$str =~ s/[\n\s]+$//;
		return $str;
	}
	sub get_requires {
		my ($tpm,$type) = @_;
		my %requires = $tpm->getHash("Requires");
		my @keylist = keys %requires;
		my @tlcorereqlist = (); 
		my @packagereqlist = ();
		if ($type eq '') { 
			$type = "all";
		}
		foreach my $k (keys %requires) {
			foreach my $e (@{$requires{$k}}) {
				# manually exclude Windows-only packages
				   if ($e eq "bin-bzip2") {}
				elsif ($e eq "bin-xpdf") {}
				elsif ($e eq "bin-chktex") {}
				elsif ($e eq "bin-ghostscript") {}
				elsif ($e eq "bin-gzip") {}
				elsif ($e eq "bin-jpeg2ps") {}
				elsif ($e eq "bin-perl") {}
				elsif ($e eq "bin-windvi") {}
				elsif ($e eq "lib-freetype2") {}
				elsif ($e eq "lib-gd") {}
				elsif ($e eq "lib-geturl") {}
				elsif ($e eq "lib-gnu") {}
				elsif ($e eq "lib-gs") {}
				elsif ($e eq "lib-jpeg") {}
				elsif ($e eq "lib-md5") {}
				elsif ($e eq "lib-png") {}
				elsif ($e eq "lib-regex") {}
				elsif ($e eq "lib-texmfmp") {}
				elsif ($e eq "lib-tiff") {}
				elsif ($e eq "lib-ttf") {}
				elsif ($e eq "lib-xpdf") {}
				elsif ($e eq "lib-xpm") {}
				elsif ($e eq "lib-zlib") {}
				elsif ($e  =~ /^bin-(.*)$/) {
					push @packagereqlist, "$k/$e";
				}
				elsif ($e  =~ /^hyphen-(.*)$/){
					push @packagereqlist, "$k/$e";
				}
				elsif ($k eq "TLCore") {
					push @tlcorereqlist, "$k/$e";
				} else {
					push @packagereqlist, "$k/$e";
				}
			}
		}
		if ($type eq '' || $type eq 'all') {
			my %foo;
			$foo{'TLCore'} = \@tlcorereqlist;
			$foo{'Package'} = \@packagereqlist;
			#my @foo=(@tlcorereqlist,@packagereqlist);
			#return(@foo);
			return(\%foo);
		} elsif ($type eq 'TLCore') {
			return(@tlcorereqlist);
		} elsif ($type eq 'Package') {
			return(@packagereqlist);
		} else {
			die("Don't know this type: $type!\n");
		}
	}
	# start of real function
	use_global_vars();
	print "Start loading tpm data ... \n";
	foreach my $t ('TLCore', 'Documentation', 'Package') {
		my $subtree = ${TexmfTreeOfType}{$t};
		foreach my $f (<$Master/$subtree/tpm/*.tpm>) {
			my $shortn = basename($f);
			$shortn =~ s,\.tpm$,,;
			if (($t eq 'TLCore') && ($shortn =~ m/^scheme-/)) { next ; }
			my $tpm = Tpm->new("$subtree/tpm/$shortn.tpm");
			$TpmData{$t}{$shortn}{'BinFiles'} = [ $tpm->getFileList("BinFiles") ];
			$TpmData{$t}{$shortn}{'DocFiles'} = [ $tpm->getFileList("DocFiles") ];
			$TpmData{$t}{$shortn}{'RunFiles'} = [ $tpm->getFileList("RunFiles") ];
			$TpmData{$t}{$shortn}{'SourceFiles'} = [ $tpm->getFileList("SourceFiles") ];
			$TpmData{$t}{$shortn}{'RemoteFiles'} = [ $tpm->getFileList("RemoteFiles") ];
			$TpmData{$t}{$shortn}{'Title'} = trim($tpm->getAttribute("Title"));
			# print "got title $TpmData{$t}{$shortn}{'Title'}\n";
			$TpmData{$t}{$shortn}{'Description'} = trim($tpm->getAttribute("Description"));
			$TpmData{$t}{$shortn}{'License'} = trim($tpm->getAttribute("License"));
			my @foo = $tpm->getList("Installation");
			$TpmData{$t}{$shortn}{'Installation'} = \@foo ;
			my $alldeps = get_requires($tpm,'all');
			my @incs = @{$alldeps->{'Package'}};
			my @deps = @{$alldeps->{'TLCore'}};
			$TpmData{$t}{$shortn}{'Package'} = \@incs;
			$TpmData{$t}{$shortn}{'TLCore'} = \@deps;
		}
	}
	print " ... done\n";
}

sub check_consistency {
	my %UsedPackages;
	my $raiseerror = 0;
	my @allincludedpkgs = ();
	my %PackageToCollection;

	print "Checking consistency ... \n";
	#
	# first go through all the collections and collect all included
	# packages and check on double inclusion of packages
	#
	my %TLB = %{$TeXLive{'binary'}};
	foreach my $collection (keys %TLB) {
		if ($TLB{$collection}{'type'} ne "TLCore") { next; }
		foreach my $tpm (@{$TLB{$collection}{'includedpackages'}}) {
			$PackageToCollection{$tpm} = $collection;
			push @allincludedpkgs, $tpm;
			push (@{$UsedPackages{$tpm}},$collection);
		}
	}
	my @badpacks = ();
	foreach my $k (keys %UsedPackages) {
		if (@{$UsedPackages{$k}} > 1) {
			print STDERR "Double inclusion of $k:\n";
			print STDERR "@{$UsedPackages{$k}}:\n";
			push @badpacks, $k;
		}
	}
	if ($#badpacks >= 0) {
		#
		# ujjeeee, a package is included more than once, STOP!!!
		#
		print ("ERROR double-inclusion @badpacks\n");
		$raiseerror = 1;
	}
	#
	# check for not included packages
	#
	my $binary_package;
	foreach $binary_package (keys %TLB) {
		if ($TLB{$binary_package}{'type'} ne "Package") { next; }
		if (grep(/$binary_package/,@allincludedpkgs) == 0) {
			print STDERR "ERROR not-covered $binary_package\n";
			$raiseerror = 1;
		}
	}
	#
	# check for packages in packages which are not included or depended on
	#
	foreach my $c (keys %TLB) {
		if ($TLB{$binary_package}{'type'} ne "TLCore") { next; }
		foreach $binary_package (@{$TLB{$c}{'includedpackages'}}) {
			foreach my $subdep (@{$TLB{$binary_package}{'includedpackages'}}) {
				# $binary_package depends on $subdep, we have to check wether
				# either $subdep is also included in $c
				# or $c depends on the collection $subdep is in
				my $subdepcoll = $PackageToCollection{$subdep};
				# the package is included in the parent collection, ok
				if ($subdepcoll eq $c) { next ; }
				# the parent collection depends on the collection
				# including the subdep by tpm depends
				if ($subdepcoll eq "texlive-base-bin") { next; }
				if (ismember($subdepcoll,@{$TLB{$c}{'depends'}})) { next; }
				# ujeee, cross wise inclusion does not work
				print STDERR "ERROR diamond\n";
				print STDERR "There is a problem here:\n";
				print STDERR "$c includes $binary_package\n";
				print STDERR "\t$binary_package depends on $subdep\n";
				print STDERR "\t$subdep is NOT included in $c\n";
				print STDERR "$subdep is included in $subdepcoll\n";
				print STDERR "$c does NOT depend on $subdepcoll\n";
				print STDERR "Please fix this problem!\n";
				$raiseerror = 1;
			}
		}
	}
	if ($raiseerror) { exit 1; }
	print " ... tpm files and config file are consistent, good!\n";
}


sub initialize_config_file_data {
	my ($cfgfile) = @_;
	use_global_vars();
	print "Start loading config file ...\n";
	open(CFGFILE,"<$cfgfile") or die "Cannot open $cfgfile\n";
	while (<CFGFILE>) {
		if (m/^#/) { 
			next ; 
		}
		chomp;
		my @foo;
		my ($type, $a, @rest) = split ";";
		$opt_debug && print STDERR  "type=$type, a=$a, ";
		if ($type eq "depends") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'depends'}{$a} = [ @{$Config{'depends'}{$a}}, split(/[ \t]*,[ \t]*/,$b) ];
			next;
		}
		if ($type eq "recommends") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'recommends'}{$a} = [ @{$Config{'recommends'}{$a}}, split(/[ \t]*,[ \t]*/,$b) ];
			next;
		}
		if ($type eq "provides") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'provides'}{$a} = [ @{$Config{'provides'}{$a}}, split(/[ \t]*,[ \t]*/,$b) ];
			next;
		}
		if ($type eq "suggests") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'suggests'}{$a} = [ @{$Config{'suggests'}{$a}}, split(/[ \t]*,[ \t]*/,$b) ];
			next;
		}
		if ($type eq "conflicts") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'conflicts'}{$a} = [ @{$Config{'conflicts'}{$a}}, split(/[ \t]*,[ \t]*/,$b) ];
			next;
		}
		if ($type eq "replaces") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'replaces'}{$a} = [ @{$Config{'replaces'}{$a}}, split(/[ \t]*,[ \t]*/,$b) ];
			next;
		}
		if ($type eq "execute") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'add_execute'}{$a} = [ @{$Config{'add_execute'}{$a}}, "$b" ];
			next;
		}
		if ($type eq "mapping") {
			my ($b,$c) = @rest;
			$opt_debug && print STDERR  "b=$b, c=$c.\n";
			push @{$TeXLive{'all'}{'filemappings'}}, "$a:$c";
			$TeXLive{'all'}{'file_map_actions'}{$a} = $b;
			if (($b eq "config-link") || ($b eq "config-remap")) {
				push @{$TeXLive{'all'}{'config-files'}}, $c;
			}
			if ($b eq "config-copy") {
				$a =~ s#texmf-dist#texmf#;
				$a =~ s#texmf-doc#texmf#;
				$a =~ s#texmf/##;
				push @{$TeXLive{'all'}{'config-files'}}, "$c/$a";
			}
			next;
		}
		if ($type eq "blacklist") {
			if ($a eq "file") {
				my ($b) = @rest;
				$opt_debug && print STDERR  "b=$b.\n";
				push @{$TeXLive{'all'}{'file_blacklist'}}, $b;
			} elsif ($a eq "tpm") {
				my ($b,$c) = @rest;
				$opt_debug && print STDERR  "b=$b, c=$c.\n";
				$Config{'package_blacklist'}{$b} = "$c";
			} else {
				print STDERR "tpm2deb.cfg: Unknown directive: $_. Exiting!\n";
				exit 1;
			}
			next;
		}
		if ($type eq "source") {
			push @{$TeXLive{'all'}{'sources'}}, $a;
			next;
		}
		if ($type eq "move") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			# move tpm $a to binary package $b
			$Config{'moved_to'}{$a} = $b;
			$Config{'extra_packages'}{$b} = [ @{$Config{'extra_packages'}{$b}}, $a ];
			next;
		}
		if ($type eq "extra") {
			my ($b,$c,$d) = @rest;
			if ($a ne "format") {
				print STDERR "tpm2deb.cfg: Unknown directive: $_. Exiting!\n";
				exit 1;
			}
			$opt_debug && print STDERR  "b=$b, c=$c, d=$d.\n";
			$TeXLive{'binary'}{$c}{'extra_format'}{$b} = $d;
			next;
		}
		if ($type eq "arch") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$TeXLive{'source'}{$a}{'architecture'} = $b;
			next;
		}
		if ($type eq "name") {
			my ($b,$c) = @rest;
			$opt_debug && print STDERR  "b=$b, c=$c.\n";
			$Config{'name_mapping'}{$a} = $b;
			$TeXLive{'binary'}{$b}{'source_package'} = $c;
			$TeXLive{'source'}{$c}{'binary_packages'} = [ @{$TeXLive{'source'}{$c}{'binary_packages'}}, $b ];
			next;
		}
		if ($type eq "special") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			push @{$TeXLive{'all'}{'special_actions_config'}}, "$a:$b";
			next;
		}
		if ($type eq "description") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			# push the next line into the description string, but
			# avoid to get a "use of uninitialized value in concat..."
			$Config{'description'}{$a} = 
				($Config{'description'}{$a} ? $Config{'description'}{$a} : "") 
				. "$b\n";
			next;
		}
		if ($type eq "title") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'title'}{$a} = "$b";
			next;
		}
		if ($type eq "docsplitting") {
			$opt_debug && print STDERR  "\n";
			$Config{'docsplitting'}{$a} = 1;
			next;
		}
		if ($type eq "build-dep") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'build_dep'}{$a} = "$b";
			next;
		}
		if ($type eq "build-dep-indep") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'build_dep_indep'}{$a} = "$b";
			next;
		}
		if ($type eq "texlive-common-version") {
			$TeXLive{'all'}{'tl_common_version'} = "$a";
			next;
		}
		if ($type eq "latest-version") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$TeXLive{'source'}{$a}{'latest_version'} = "$b";
			next;
		}
		if ($type eq "maintainer") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			if ("$a" eq "*") {
				$TeXLive{'all'}{'maintainer'} = "$b";
			} else {
				$Config{'maintainer'}{$a} = "$b";
			}
			next;
		}
		if ($type eq "priority") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			if ("$a" eq "*") {
				$TeXLive{'all'}{'priority'} = "$b";
			} else {
				$Config{'priority'}{$a} = "$b";
			}
			next;
		}
		if ($type eq "uploaders") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			if ("$a" eq "*") {
				$TeXLive{'all'}{'uploaders'} = "$b";
			} else {
				$Config{'uploaders'}{$a} = "$b";
			}
			next;
		}
		if ($type eq "section") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			if ("$a" eq "*") {
				$TeXLive{'all'}{'section'} = "$b";
			} else {
				$Config{'section'}{$a} = "$b";
			}
			next;
		}
		if ($type eq "bin-section") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			$Config{'bin-section'}{$a} = "$b";;
			next;
		}
		if ($type eq "standards") {
			my ($b) = @rest;
			$opt_debug && print STDERR  "b=$b.\n";
			if ("$a" eq "*") {
				$TeXLive{'all'}{'standards'} = "$b";
			} else {
				$Config{'standards'}{$a} = "$b";
			}
			next;
		}
		if ($type eq "addpackages") {
			@{$Config{'add_packages'}} = @rest;
			$opt_debug && print STDERR "\nAdditional packages: @{$Config{'add_packages'}}\n";
			next;
		}
		if ($type eq "dependsnot") {
			@{$Config{'depends_not'}{$a}} = @rest;
			$opt_debug && print STDERR "Dropped depends of $a on @{$Config{'depends_not'}{$a}}\n";
			next;
		}
		print STDERR "tpm2deb.cfg: Unknown directive: $type. Exiting!\n"; 
		exit 1;
	}
	close(CFGFILE);
	print " ... done\n";
}


sub is_blacklisted {
	my ($tpm,$coll) = @_;
	# blacklist check
	# either there is no blacklist entry, then it is ok
	# if there is a blacklist entry and it is *, blacklist it
	# if it is not *, and the $coll is not "", then match it
	if (defined($Config{'package_blacklist'}{$tpm}) &&
			(($Config{'package_blacklist'}{$tpm} eq "*") ||
			 (($coll ne "") && ($Config{'package_blacklist'}{$tpm} =~ m/$coll/)))) {
		return(1);
	} else {
		return(0);
	}
}

sub get_all_files {
	my ($entry,$reclevel) = @_;
	my @requires = @{$TeXLive{'binary'}{$entry}{'includedpackages'}};
	my %files;
	#
	$files{'BinFiles'} = \@{$TeXLive{'binary'}{$entry}{'binfiles'}};
	$files{'DocFiles'} = \@{$TeXLive{'binary'}{$entry}{'docfiles'}};
	$files{'RunFiles'} = \@{$TeXLive{'binary'}{$entry}{'runfiles'}};
	$files{'SourceFiles'} = \@{$TeXLive{'binary'}{$entry}{'sourcefiles'}};
	$files{'RemoteFiles'} = \@{$TeXLive{'binary'}{$entry}{'remotefiles'}};
	if ($reclevel > 0) {
		foreach my $r (@requires) {
			$opt_debug && print STDERR  "  package " . $r . "\n";
			my %foo = %{&get_all_files($r,$reclevel-1)};
			push @{$files{'BinFiles'}}, @{$foo{'BinFiles'}};
			push @{$files{'DocFiles'}}, @{$foo{'DocFiles'}};
			push @{$files{'RunFiles'}}, @{$foo{'RunFiles'}};
			push @{$files{'SourceFiles'}}, @{$foo{'SourceFiles'}};
			push @{$files{'RemoteFiles'}}, @{$foo{'RemoteFiles'}};
		}
	}
	return(\%files);
}

sub get_all_executes {
	my ($entry,$reclevel) = @_;
	my @requires = @{$TeXLive{'binary'}{$entry}{'includedpackages'}};
	my %bar;
	my @executes;
	#
	@executes = @{$TeXLive{'binary'}{$entry}{'executes'}};
	if ($reclevel > 0) {
		foreach my $r (@requires) {
			$opt_debug && print STDERR  "  package " . $r . "\n";
			my @foo = get_all_executes($r,$reclevel-1);
			push @executes, @foo;
		}
	}
	foreach (@executes) {
		$bar{$_} = 1;
	}
	return(keys %bar);
}
 
sub myopen {
	my ($a, $fn) = @_;
	open($a, $fn) or die "Cannot open $fn: $!\n";
}

sub mycopy {
	my ($src,$dest) = @_;
	&mkpath(dirname($dest));
# 	system("/bin/cp -a $src $dest 2>/dev/null") == 0
	my $cp_cmdline = "/bin/cp -a $src $dest" . ( $opt_debug ? "" : " 2>/dev/null");
	if ($opt_onlyscripts == 0) {
		system($cp_cmdline) == 0
			or do {
				print STDERR "$cp_cmdline\n";
				die ("missing: $src\n");
		};
	};
}

sub ismember {
	my ($a, @rest) = @_;
	foreach my $i (@rest) {
		if ($a eq $i) { return 1; }
	}
	return 0;
}

#####################################
sub dump_tpm_data {
	print "Dumping TpmData\n\n";
	foreach my $t ('TLCore', 'Documentation', 'Package') {
		print "Dumping $t:\n";
		my %foo = %{$TpmData{$t}};
		foreach my $p (keys %foo) {
			print "$p:\n";
			print "\tbinfiles: @{$TpmData{$t}{$p}{'BinFiles'}}<<<\n";
			print "\trunfiles: @{$TpmData{$t}{$p}{'RunFiles'}}<<<\n";
			print "\tdocfiles: @{$TpmData{$t}{$p}{'DocFiles'}}<<<\n";
			print "\tsourcefiles: @{$TpmData{$t}{$p}{'SourceFiles'}}<<<\n";
			print "\tremotefiles: @{$TpmData{$t}{$p}{'RemoteFiles'}}<<<\n";
			print "\ttitle: $TpmData{$t}{$p}{'Title'}\n";
			print "\tDescription: $TpmData{$t}{$p}{'Description'}\n";
			print "\tLicense: $TpmData{$t}{$p}{'License'}\n";
			print "\tInstallation: @{$TpmData{$t}{$p}{'Installation'}}\n";
			print "\tDep-Package: @{$TpmData{$t}{$p}{'Package'}}\n";
			print "\tDep-TLCore: @{$TpmData{$t}{$p}{'TLCore'}}\n";
		}
	}
}

sub store_tpm {
	my ($fn) = @_;
	print "Storing TpmData into $fn ...\n";
	store(\%TpmData, $fn) or die "Can't store TpmData in $fn!\n";
	print "  ...done\n";
}


1;

### Local Variables:
### perl-indent-level: 4
### tab-width: 4
### indent-tabs-mode: t
### End:
# vim:set tabstop=4: #
